
' engine.bas

'===============================================================================
' Author ............ Jrgen Schlottke, Schnaich-C.-Str. 46, D-W 2200 Elmshorn
' Purpose ............... Demonstration of chess programming under Turbo Pascal
' Date of release .................................................... 18.01.93
' FreeBASIC adaptation ........... Roland Chastain, Sarreguemines, France, 2014
'===============================================================================

#include "flag.bas"

#define whiteColor +1
#define blackColor -1
#define EOL chr(13, 10)
#define none 0
#define maxListIndex 80

const as byte cNothingValue = 000
const as byte cPawnValue    = 002
const as byte cBishopValue  = 006
const as byte cKnightValue  = 007
const as byte cRookValue    = 010
const as byte cQueenValue   = 019
const as byte cKingValue    = 126
const as byte cBorderValue  = 127

dim shared as byte cPawnMove  (1 to 4) = {+01, +02, -09, +11}
dim shared as byte cBishopMove(1 to 4) = {+11, -11, +09, -09}
dim shared as byte cRookMove  (1 to 4) = {-01, +01, -10, +10}
dim shared as byte cKnightMove(1 to 8) = {+12, +21, +19, +08, -12, -21, -19, -08}
dim shared as byte cKingMove  (1 to 8) = {-01, +01, -10, +10, +11, -11, +09, -09}

#ifdef tBoolean
rem Declared in chess.bas
#else
enum tBoolean
  isTrue  = (0 = 0)
  isFalse = (0 > 0)
end enum
#endif

#ifdef tCastling
rem Declared in chess.bas
#else
enum tCastling
  e1g1
  e1c1
  e8g8
  e8c8
end enum
#endif

type tMoveRec
  as byte fromSquare
  as byte toSquare
  as short value
end type

type tMoveList
  as tMoveRec array(1 to maxListIndex)
  as byte count
end type

type tBoard
  as byte board(-10 to 109)
  as byte castlingAvailability(e1g1 to e8c8)
  as byte castlingLicence(e1g1 to e8c8)
  as byte enPassantTargetSquare(blackColor to whiteColor)
  as byte activeColor, oneKingCaptured
  as short halfmoveCount, materialBalance
  as tMoveList list
  declare sub Initialize(byval as string = "")
  declare sub MovePiece(byval as byte, byval as byte, byval as byte = cQueenValue)
  declare sub AddMoveToList(byval as byte, byval as byte)
  declare sub GenerateMoves()
  declare sub GenerateCastling(byval as tCastling)
  declare sub GenerateAllMoves()
  declare sub CheckCastlingLicence(byval as byte)
  declare function PositionValue(byval as byte) as short
  declare function Text() as string
  declare function HtmlText as string
  declare function Check() as byte
end type

type tChessEngine
  as byte minDepth = 3
  as byte maxDepth = 5
  as tBoard playBoard, startBoard
  as tMoveList list
  declare sub Initialize(byval as string = "")
  declare function RecursiveEvaluation(byref as tBoard, byval as byte, byval as byte, byval as short) as short
  declare function UserMove(byval as string, byval as byte = cQueenValue) as integer
  declare function InitialState(byref as tBoard, byval as byte) as byte
  declare function BestMove(byval as short) as byte
  declare function EngineMove(byref as byte) as string
end type

function MoveToStr(byval aFromSquare as byte, byval aToSquare as byte) as string
  dim result as string = str(aFromSquare * 100 + aToSquare)
  result[0] += asc("a") - asc("1") ' 48
  result[2] += asc("a") - asc("1") ' 48
  return result
end function ' MoveToStr

function HtmlBegin( _
  font as string = "chess alfonso-x", _
  size as string = "32px", _
  colour as string = "midnightblue" _
) as string
  return _
  "<!DOCTYPE html>" & EOL & _
  "<html>" & EOL & _
  "<head>" & EOL & _
  "  <title>JSCP</title>" & EOL & _
  "  <style type=""text/css"">" & EOL & _
  "  p {" & EOL & _
  "    font-family: " & font & ";" & EOL & _
  "    font-size: " & size & ";" & EOL & _
  "    color: " & colour & ";" & EOL & _
  "  }" & EOL & _
  "  </style>" & EOL & _
  "</head>" & EOL & _
  "<body>"
end function ' HtmlBegin

function HtmlEnd as string
  return _
  "</body>" & EOL & _
  "</html>"
end function ' HtmlEnd

sub tBoard.Initialize(byval s as string = "")
  dim as byte x, y, z
  for z = -10 to 109
    board(z) = cBorderValue
  next z
  if len(s) = 0 then
    restore defaultPlacement
    for y = 8 to 1 step -1
      for x = 1 to 8
        read board(10 * x + y)
      next x
    next y
    castlingAvailability(e1g1) = isTrue
    castlingAvailability(e1c1) = isTrue
    castlingAvailability(e8g8) = isTrue
    castlingAvailability(e8c8) = isTrue
    enPassantTargetSquare(whiteColor) = none
    enPassantTargetSquare(blackColor) = none
    materialBalance = 0
    activeColor = whiteColor
    halfmoveCount = 0
  else
    dim as string symbol = "kqrbnp PNBRQK", a(1 to 6)
    z = instr(s, " ") : a(1) = left(s, z - 1) : s = right(s, len(s) - z)
    z = instr(s, " ") : a(2) = left(s, z - 1) : s = right(s, len(s) - z)
    z = instr(s, " ") : a(3) = left(s, z - 1) : s = right(s, len(s) - z)
    z = instr(s, " ") : a(4) = left(s, z - 1) : s = right(s, len(s) - z)
    z = instr(s, " ") : a(5) = left(s, z - 1) : s = right(s, len(s) - z)
    a(6) = s
    
    x = 1
    y = 8
    for z = 1 to len(a(1))
      if a(1)[z - 1] = asc("/") then
        x = 1
        y -= 1
      end if
      if (a(1)[z - 1] >= asc("1")) andalso (a(1)[z - 1] <= asc("8")) then
        do while a(1)[z - 1] > asc("0")
          board(10 * x + y) = cNothingValue
          a(1)[z - 1] -= 1
          x += 1
        loop
      end if
      if instr(symbol, chr(a(1)[z - 1])) > 0 then
        select case instr(symbol, chr(a(1)[z - 1])) - 7
          case -6 : board(10 * x + y) = blackColor * cKingValue
          case -5 : board(10 * x + y) = blackColor * cQueenValue
          case -4 : board(10 * x + y) = blackColor * cRookValue
          case -3 : board(10 * x + y) = blackColor * cBishopValue
          case -2 : board(10 * x + y) = blackColor * cKnightValue
          case -1 : board(10 * x + y) = blackColor * cPawnValue
          case +1 : board(10 * x + y) = whiteColor * cPawnValue
          case +2 : board(10 * x + y) = whiteColor * cKnightValue
          case +3 : board(10 * x + y) = whiteColor * cBishopValue
          case +4 : board(10 * x + y) = whiteColor * cRookValue
          case +5 : board(10 * x + y) = whiteColor * cQueenValue
          case +6 : board(10 * x + y) = whiteColor * cKingValue
        end select
        x += 1
      end if
    next z
    for y = 8 to 1 step -1
      for x = 1 to 8
        materialBalance += board(10 * x + y)
      next x
    next y
    activeColor = iif(a(2) = "w", whiteColor, blackColor)
    castlingAvailability(e1g1) = (instr(a(3), "K") > 0)
    castlingAvailability(e1c1) = (instr(a(3), "Q") > 0)
    castlingAvailability(e8g8) = (instr(a(3), "k") > 0)
    castlingAvailability(e8c8) = (instr(a(3), "q") > 0)
    if len(a(4)) = 2 then
      enPassantTargetSquare(-1 * activeColor) = 10 * (a(4)[0] - 96) + (a(4)[1] - 48)
    else
      enPassantTargetSquare(-1 * activeColor) = none
    end if
    enPassantTargetSquare(activeColor) = none
    halfmoveCount = 2 * (valint(a(6)) - 1) + iif(activeColor = blackColor, 1, 0)
  end if
  oneKingCaptured = isFalse
end sub ' Initialize

sub tBoard.MovePiece( _
  byval aFromSquare as byte, _
  byval aToSquare as byte, _
  byval aPromotion as byte = cQueenValue _
)
  if abs(board(aFromSquare)) = cKingValue then
    if sgn(board(aFromSquare)) = whiteColor then
      castlingAvailability(e1g1) = isFalse
      castlingAvailability(e1c1) = isFalse
    else
      castlingAvailability(e8g8) = isFalse
      castlingAvailability(e8c8) = isFalse
    end if
  end if
  
  if abs(board(aFromSquare)) = cRookValue then
    select case aFromSquare
      case 81 : castlingAvailability(e1g1) = isFalse
      case 11 : castlingAvailability(e1c1) = isFalse
      case 88 : castlingAvailability(e8g8) = isFalse
      case 18 : castlingAvailability(e8c8) = isFalse
    end select
  end if
  
  if abs(board(aToSquare)) = cRookValue then
    select case aToSquare
      case 81 : castlingAvailability(e1g1) = isFalse
      case 11 : castlingAvailability(e1c1) = isFalse
      case 88 : castlingAvailability(e8g8) = isFalse
      case 18 : castlingAvailability(e8c8) = isFalse
    end select
  end if
  
  if abs(board(aToSquare)) = cKingValue then oneKingCaptured = isTrue
  
  if (abs(aToSquare - aFromSquare) = 2) and (abs(board(aFromSquare)) = cPawnValue) then
    enPassantTargetSquare(activeColor) = aFromSquare + cPawnMove(1) * activeColor
  else
    enPassantTargetSquare(activeColor) = none
  end if
  
  if (aToSquare = enPassantTargetSquare(- 1 * activeColor)) and (board(aFromSquare) = cPawnValue * activeColor) then
     enPassantTargetSquare(activeColor) = none
     MovePiece(aFromSquare, (aToSquare \ 10) * 10 + aFromSquare mod 10)
     activeColor = -1 * activeColor
     halfmoveCount -= 1
     aFromSquare = (aToSquare \ 10) * 10 + aFromSquare mod 10
  end if
  
  if ((aFromSquare >= 51) and (aFromSquare <= 58)) _
  andalso ( _
    (aToSquare = 71) or _
    (aToSquare = 31) or _
    (aToSquare = 78) or _
    (aToSquare = 38) _
  ) _
  andalso (abs(board(aFromSquare)) = cKingValue) then
    board(aToSquare) = board(aFromSquare)
    board(aFromSquare) = cNothingValue
    
    if aToSquare \ 10 = 7 then
      aFromSquare = (aFromSquare mod 10) + 80
      aToSquare = aFromSquare - 20
    else
      aFromSquare = (aFromSquare mod 10) + 10
      aToSquare = aFromSquare + 30
    end if
    
    board(aToSquare) = board(aFromSquare)
    board(aFromSquare) = cNothingValue
  else
    materialBalance = materialBalance - board(aToSquare)
    board(aToSquare) = board(aFromSquare)
    board(aFromSquare) = cNothingValue
    if ( _
      (aToSquare mod 10 = 1) or _
      (aToSquare mod 10 = 8) _
    ) and (abs(board(aToSquare)) = cPawnValue) then
      board(aToSquare) = aPromotion * activeColor
      materialBalance = materialBalance + (aPromotion - cPawnValue) * activeColor
    end if
  end if
  
  activeColor = -1 * activeColor
  halfmoveCount += 1
end sub ' MovePiece

sub tBoard.AddMoveToList(byval aFromSquare as byte, byval aToSquare as byte)
  if (oneKingCaptured andalso abs(board(aToSquare)) = cKingValue) _
  or (list.count = maxListIndex) then exit sub
  
  list.count += 1
  list.array(list.count).fromSquare = aFromSquare
  list.array(list.count).toSquare = aToSquare
end sub ' AddMoveToList

sub tBoard.GenerateMoves()
  dim as integer aFromSquare, aToSquare
  dim as integer i, j, k, l
  
  list.count = 0
  
  for j = 1 to 8
    for k = 1 to 8
      aFromSquare = j * 10 + k
      if board(aFromSquare) <> cNothingValue then
        select case board(aFromSquare) * activeColor
        
        case cPawnValue
          aToSquare = aFromSquare + cPawnMove(1) * activeColor
          
          if board(aToSquare) = cNothingValue then
            AddMoveToList(aFromSquare, aToSquare)
            if (activeColor = whiteColor) and (k = 2) _
            or (activeColor = blackColor) and (k = 7) then
              aToSquare = aFromSquare + cPawnMove(2) * activeColor
              if (board(aToSquare) = cNothingValue) then
                AddMoveToList(aFromSquare, aToSquare)
              end if
            end if
          end if
          
          for i = 3 to 4
            aToSquare = aFromSquare + cPawnMove(i) * activeColor
            if ((-1 * board(aToSquare) * activeColor >= cPawnValue) _
            and (-1 * board(aToSquare) * activeColor <= cKingValue)) _
            or (aToSquare = enPassantTargetSquare(-1 * activeColor)) then
              AddMoveToList(aFromSquare, aToSquare)
            end if
          next i
          
        case cKnightValue
          for i = 1 to 8
            aToSquare = aFromSquare + cKnightMove(i)
            if (-1 * board(aToSquare) * activeColor >= cNothingValue) _
            and (-1 * board(aToSquare) * activeColor <= cKingValue) then
              AddMoveToList(aFromSquare, aToSquare)
            end if
          next i
          
        case cBishopValue
          for i = 1 to 4
            aToSquare = aFromSquare
            do
              aToSquare += cBishopMove(i)
              if (-1 * board(aToSquare) * activeColor >= cNothingValue) _
              and (-1 * board(aToSquare) * activeColor <= cKingValue) then
                AddMoveToList(aFromSquare, aToSquare)
              end if
            loop until board(aToSquare) <> cNothingValue
          next i
          
        case cRookValue
          for i = 1 to 4
            aToSquare = aFromSquare
            do
              aToSquare += cRookMove(i)
              if (-1 * board(aToSquare) * activeColor >= cNothingValue) _
              and (-1 * board(aToSquare) * activeColor <= cKingValue) then
                AddMoveToList(aFromSquare, aToSquare)
              end if
            loop until board(aToSquare) <> cNothingValue
          next i
          
        case cKingValue
          for i = 1 to 8
            aToSquare = aFromSquare + cKingMove(i)
            if (-1 * board(aToSquare) * activeColor >= cNothingValue) _
            and (-1 * board(aToSquare) * activeColor <= cKingValue) then
              l = 0
              do
                l += 1
              loop until (-1 * board(aToSquare + cKingMove(l)) * activeColor = cKingValue) or (l = 8)
              if l = 8 then AddMoveToList(aFromSquare, aToSquare)
            end if
          next i
          
        case cQueenValue
          for i = 1 to 8
            aToSquare = aFromSquare
            do
              aToSquare += cKingMove(i)
              if (-1 * board(aToSquare) * activeColor >= cNothingValue) _
              and (-1 * board(aToSquare) * activeColor <= cKingValue) then
                AddMoveToList(aFromSquare, aToSquare)
              end if
            loop until board(aToSquare) <> cNothingValue
          next i
        end select
      end if
    next k
  next j
end sub ' GenerateMoves

sub tBoard.GenerateCastling(byval aCastling as tCastling)
  dim as byte kingFrom, kingStep, rookFrom, i
  
  select case as const aCastling
    case e1g1
      kingFrom = 51
      kingStep = +10
      rookFrom = 81
    case e1c1
      kingFrom = 51
      kingStep = -10
      rookFrom = 11
    case e8g8
      kingFrom = 58
      kingStep = +10
      rookFrom = 88
    case e8c8
      kingFrom = 58
      kingStep = -10
      rookFrom = 18
  end select
  
  i = kingFrom + kingStep
  do
    if board(i) <> cNothingValue then exit sub
    i += kingStep
  loop until i = rookFrom
  AddMoveToList(kingFrom, kingFrom + 2 * kingStep)
end sub

sub tBoard.GenerateAllMoves()
  CheckCastlingLicence(activeColor)
  GenerateMoves()
  
  if activeColor = whiteColor then
    if castlingAvailability(e1g1) andalso castlingLicence(e1g1) then GenerateCastling(e1g1)
    if castlingAvailability(e1c1) andalso castlingLicence(e1c1) then GenerateCastling(e1c1)
  else
    if castlingAvailability(e8g8) andalso castlingLicence(e8g8) then GenerateCastling(e8g8)
    if castlingAvailability(e8c8) andalso castlingLicence(e8c8) then GenerateCastling(e8c8)
  end if
end sub ' GenerateAllMoves

sub tBoard.CheckCastlingLicence(byval aColor as byte)
  dim c as tCastling
  dim y as byte = iif(aColor = whiteColor, 1, 8)
  for c = e1g1 to e8c8
    castlingLicence(c) = isTrue
  next c
  
  activeColor = -1 * activeColor
  GenerateMoves()
  
  activeColor = -1 * activeColor
  for i as integer = 1 to list.count
    if list.array(i).toSquare mod 10 = y then
      select case list.array(i).toSquare \ 10
        case is = 5
          castlingLicence(e1g1) = isFalse
          castlingLicence(e1c1) = isFalse
          castlingLicence(e8g8) = isFalse
          castlingLicence(e8c8) = isFalse
        case is > 5
          castlingLicence(e1g1) = isFalse
          castlingLicence(e8g8) = isFalse
        case is < 5
          castlingLicence(e1c1) = isFalse
          castlingLicence(e8c8) = isFalse
      end select
    end if
  next i
end sub ' CheckCastlingLicence

function tBoard.PositionValue(byval aColor as byte) as short
  return materialBalance * aColor
end function ' PositionValue

function tBoard.Text() as string
  dim as string s = _
  "+ A B C D E F G H " & EOL & _
  "8                 " & EOL & _
  "7                 " & EOL & _
  "6                 " & EOL & _
  "5                 " & EOL & _
  "4                 " & EOL & _
  "3                 " & EOL & _
  "2                 " & EOL & _
  "1                 " & EOL & _
  "+ A B C D E F G H "
  for y as integer = 8 to 1 step -1
    for x as integer = 1 to 8
      select case abs(board(10 * x + y))
        case cNothingValue
          s[20 * (8 - y) + 2 * x + 20] = asc("-")
        case cPawnValue
          s[20 * (8 - y) + 2 * x + 20] = asc("P")
        case cKnightValue
          s[20 * (8 - y) + 2 * x + 20] = asc("N")
        case cBishopValue
          s[20 * (8 - y) + 2 * x + 20] = asc("B")
        case cRookValue
          s[20 * (8 - y) + 2 * x + 20] = asc("R")
        case cQueenValue
          s[20 * (8 - y) + 2 * x + 20] = asc("Q")
        case cKingValue
          s[20 * (8 - y) + 2 * x + 20] = asc("K")
      end select
      if board(10 * x + y) < cNothingValue then
        s[20 * (8 - y) + 2 * x + 20] += 32
      end if
    next x
  next y
  return s
end function ' Text

function tBoard.HtmlText as string
  dim as string char, s = _
  "<p>" & EOL & _
  "!""""""""""""""""#<br>" & EOL & _
  "$tMvWlVmT%<br>" & EOL & _
  "$OoOoOoOo%<br>" & EOL & _
  "$ + + + +%<br>" & EOL & _
  "$+ + + + %<br>" & EOL & _
  "$ + + + +%<br>" & EOL & _
  "$+ + + + %<br>" & EOL & _
  "$pPpPpPpP%<br>" & EOL & _
  "$RnBqKbNr%<br>" & EOL & _
  "/(((((((()<br>" & EOL & _
  "</p>"
  for x as integer = 1 to 8
    for y as integer = 1 to 8
      select case board(10 * x + y)
        case cNothingValue             : char = " "
        case whiteColor * cPawnValue   : char = "p"
        case whiteColor * cKnightValue : char = "n"
        case whiteColor * cBishopValue : char = "b"
        case whiteColor * cRookValue   : char = "r"
        case whiteColor * cQueenValue  : char = "q"
        case whiteColor * cKingValue   : char = "k"
        case blackColor * cPawnValue   : char = "o"
        case blackColor * cKnightValue : char = "m"
        case blackColor * cBishopValue : char = "v"
        case blackColor * cRookValue   : char = "t"
        case blackColor * cQueenValue  : char = "w"
        case blackColor * cKingValue   : char = "l" 
      end select
      if (x + y) mod 2 = 0 then
        if board(10 * x + y) = cNothingValue then
          char = "+"
        else
          char = chr(asc(char) - (asc("a") - asc("A")))
        end if
      end if
      s[x + 16 * (8 - y) + 22 - 1] = asc(char)
    next y
  next x
  return s
end function

function tBoard.Check() as byte
  activeColor = -1 * activeColor
  GenerateMoves()
  
  activeColor = -1 * activeColor
  for i as integer = 1 to list.count
    if abs(board(list.array(i).toSquare)) = cKingValue then
      return isTrue
    end if
  next i
  
  return isFalse
end function ' Check

sub tChessEngine.Initialize(byval s as string = "")
  startBoard.Initialize("")
  playBoard.Initialize(s)
end sub ' Initialize

function tChessEngine.RecursiveEvaluation( _
  byref aBoard as tBoard, _
  byval aColor as byte, _
  byval aDepth as byte, _
  byval aAlpha as short _
) as short

  dim as byte i, halt
  dim as short beta, value
  dim as tBoard brd

  aBoard.GenerateAllMoves()
  
  beta = -32000 * aBoard.activeColor * aColor
  
  i = 0
  halt = isFalse
  while (i < aBoard.list.count) and not halt
    i += 1
    brd = aBoard
    brd.MovePiece(aBoard.list.array(i).fromSquare, aBoard.list.array(i).toSquare)
    
    if ( _
      (aDepth >= minDepth) and _
      (aBoard.board(aBoard.list.array(i).toSquare) = cNothingValue) _
    ) _
    or (aDepth = maxDepth) then
      value = brd.PositionValue(aColor)
    else
      value = RecursiveEvaluation(brd, aColor, aDepth + 1, beta)
    end if
    
    if aBoard.activeColor = aColor then
      if value > beta then beta = value
      if beta > aAlpha then halt = isTrue
    else
      if value < beta then beta = value
      if beta < aAlpha then halt = isTrue
    end if
    
    aBoard.list.array(i).value = value
  wend
  
  return beta
end function ' RecursiveEvaluation

function tChessEngine.UserMove( _
  byval arg as string, _
  byval aPromotion as byte = cQueenValue _
) as integer

  dim as byte fromSquare, toSquare
  dim as integer result, move
  dim brd as tBoard
  dim aux as string = arg
  
  if (len(arg) = 4) andalso _
  (arg[0] >= asc("a")) and (arg[0] <= asc("h")) and _
  (arg[1] >= asc("1")) and (arg[1] <= asc("8")) and _
  (arg[2] >= asc("a")) and (arg[2] <= asc("h")) and _
  (arg[3] >= asc("1")) and (arg[3] <= asc("8")) then
    aux[0] -= (16 + 32)
    aux[2] -= (16 + 32)
  else
    return 0
  end if
  
  result = isFalse
  move = valint(aux)
  playBoard.GenerateAllMoves()
  for i as integer = 1 to playBoard.list.count
    if move = playBoard.list.array(i).fromSquare * 100 + playBoard.list.array(i).toSquare then
      result = isTrue
      exit for
    end if
  next i
  if result = isFalse then return result
  
  fromSquare = move \ 100
  toSquare = move mod 100
  brd = playBoard
  brd.MovePiece(fromSquare, toSquare)
  brd.activeColor = -1 * brd.activeColor
  if brd.Check() then
    result = isFalse
  else
    playBoard.MovePiece(fromSquare, toSquare, aPromotion)
    playBoard.halfmoveCount += 1
  end if
  
  return result
end function ' UserMove

function tChessEngine.InitialState(byref aBoard as tBoard, byval aSquare as byte) as byte
  if aBoard.board(aSquare) = startBoard.board(aSquare) then
    return isTrue
  else
    return isFalse
  end if
end function

function tChessEngine.BestMove(byval aRecursiveEvaluation as short) as byte
  dim brd as tBoard
  dim as byte i, j
  dim as byte piece
  dim as integer max
  dim as byte index
  
  list.count = 0
  for i = 1 to playBoard.list.count
    if playBoard.list.array(i).value = aRecursiveEvaluation then
      list.count += 1
      list.array(list.count).fromSquare = playBoard.list.array(i).fromSquare
      list.array(list.count).toSquare = playBoard.list.array(i).toSquare
      list.array(list.count).value = 0
    end if
  next i
  
  max = 0
  index = 1
  for i = 1 to list.count
    brd = playBoard
    
    with list.array(i)
      .value = 0
      piece = abs(brd.board(.fromSquare))
      
      if InitialState(brd, .fromSquare) then
        .value += 5
        if piece = cPawnValue then .value += 2
      end if
      
      if (piece = cKingValue) then .value -= 10
      
      if (brd.halfmoveCount < 32) then
        if (piece = cPawnValue) _
        or (piece = cBishopValue) _
        or (piece = cKnightValue) then .value += 20
      end if
      
      if (.fromSquare \ 10 = 1) _
      or (.fromSquare \ 10 = 8) _
      or (.fromSquare mod 10 = 1) _
      or (.fromSquare mod 10 = 8) _
      then .value += 2
      
      if (.toSquare \ 10 = 1) _
      or (.toSquare \ 10 = 8) _
      or (.toSquare mod 10 = 1) _
      or (.toSquare mod 10 = 8) _
      then .value -= 2
      
      brd = playBoard
      brd.MovePiece(.fromSquare, .toSquare)
      if InitialState(brd, .toSquare) then .value -= 10
      brd.activeColor = -1 * brd.activeColor
      brd.GenerateMoves()
      
      for j = 1 to brd.list.count
        .value += 1
        if brd.board(brd.list.array(j).toSquare) <> cNothingValue then .value += 1
      next j
      
      brd.activeColor = -1 * brd.activeColor
      brd.GenerateMoves()
      
      for j = 1 to brd.list.count
        .value -= 1
        if brd.board(brd.list.array(j).toSquare) <> cNothingValue then .value -= 1
      next j
      
      if .value >= max then
        max = .value
        index = i
      end if
    end with
  next i
  
  return index
end function ' BestMove

function tChessEngine.EngineMove(byref aFlag as byte) as string
  dim as byte i
  dim as byte alreadyInCheck
  dim as string result 
  
  i = BestMove(RecursiveEvaluation(playBoard, playBoard.activeColor, 1, 32000))
  playBoard.halfmoveCount += 1
  alreadyInCheck = playBoard.Check()
  
  playBoard.MovePiece(list.array(i).fromSquare, list.array(i).toSquare)
  result = MoveToStr(list.array(i).fromSquare, list.array(i).toSquare)
  
  playBoard.activeColor = -1 * playBoard.activeColor
  if playBoard.Check() then
    if alreadyInCheck then
      aFlag = checkmateFlag
    else
      aFlag = stalemateFlag
    end if
  else
    playBoard.activeColor = -1 * playBoard.activeColor
    if playBoard.Check() then
      aFlag = checkFlag
    else
      aFlag = noFlag
    end if
  end if
  
  return result
end function ' EngineMove

defaultPlacement:
data -010, -007, -006, -019, -126, -006, -007, -010
data -002, -002, -002, -002, -002, -002, -002, -002
data -000, -000, -000, -000, -000, -000, -000, -000
data -000, -000, -000, -000, -000, -000, -000, -000
data -000, -000, -000, -000, -000, -000, -000, -000
data -000, -000, -000, -000, -000, -000, -000, -000
data +002, +002, +002, +002, +002, +002, +002, +002
data +010, +007, +006, +019, +126, +006, +007, +010
